require 'pathname'
module PDFService
  class PhantomJSRenderer
    RASTERIZE_SCRIPT_PATH = Pathname(__FILE__).dirname.join('phantomjs_rasterize.js').to_s.freeze

    def initialize(script: RASTERIZE_SCRIPT_PATH, logger: NullLogger.new, binary: 'phantomjs')
      @script = script.freeze
      @logger = logger
      @binary = binary.freeze
      Kernel.warn("Warning! PhantomJS is not available in PATH #{@binary}") if `which #{binary}`.strip.empty?
    end
    attr_reader :logger

    def rasterize(url, output, format: 'A4')
      options = [url, output, format]
      phantomjs(@script, url, options)
    end

    private
    def phantomjs(script_path, _, options = [])
      command = Shellwords.join([@binary, script_path, *options.map(&:to_s)])
      logger.debug('pdf renderer command: ' << command)
      system(command) || raise(::PDFService::Error.new("could not render PDF file. phantomjs exited with exit code = 0"))
    end
  end
end