require 'sinatra/base'
require 'logger'
require 'uri'

module PDFService
  class WebApp < Sinatra::Base
    def initialize(pdf_service:)
      super()
      @pdf_service = pdf_service
    end

    attr_reader :pdf_service
    VALID_SCHEMES = %w(http https).freeze

    get '/' do
      [200, {'Content-Type' => 'text/plain'}, ["PDFService #{PDFService::VERSION}"]]
    end


    post('/render') {
      unless !!params[:data] ^ !!params[:url]
        return ErrorResponse("one of the params :url and :data needs to be present", status: 400)
      end
      begin
        if params[:data]
          PDFResponse(@pdf_service.render(params[:data]))
        elsif url = params[:url].to_s
          url = URI.parse(url.to_s)

          unless VALID_SCHEMES.include?(url.scheme)
            return ErrorResponse("Invalid url scheme #{url.scheme}://. Valid schemes is #{VALID_SCHEMES}", status: 400)
          end
          PDFResponse(pdf_service.render_url(url))
        end
      rescue => ex
        STDERR.puts(ex.inspect)
        ErrorResponse('Internal Server Error')
      end
    }

    def ErrorResponse(message, status: 500)
      [status, {'Content-Type' => 'text/plain'}, [message]]
    end

    def PDFResponse(pdf_data, status: 200)
      [status, {'Content-Type' => 'application/pdf'}, [pdf_data]]
    end
  end
end